/*
 * RADAR - PM - 2017
 * Stan Gabriel 335CC
 */

#define F_CPU 16000000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

//#define SERVO_TEST
#define SENSOR_TEST

void IO_init() 
{
	/* Setez pinul de servo PC0 ca iesire. */
	DDRC |= (1 << PC0);
	/* Setez TRIG pt senzor ca iesire */
	DDRA |= (1 << PA1);
	/* Setez ECHO pt senzor ca intrare */
	DDRA &= ~(1 << PA0);	

	#ifndef SERVO_TEST
	/* Ledul ca iesire */
	DDRD |= (1 << PD7);
	/* Sting ledul */
	PORTD &= ~(1 << PD7);
	#endif

	/* Setez pinul de servo pe low */
	PORTC &= ~(1 << PC0);
	/* Setez TRIG pe low */
	PORTA &= ~(1 << PA1);

	/* Setez servo motor poz pe 90 */
	for (int i = 0 ; i < 100 ; i++) {
		PORTC |= (1 << PC0);
		_delay_ms(2.6);
		PORTC &= ~(1 << PC0);
		_delay_ms(18);
	}
}

void timer1_init()
{
	ICR1 = 0xFFFF;

	/* Timer 1 este configurat in modul FastPWM, top ICR1 */
	TCCR1B |= (1 << WGM12) | (1 << WGM13);
	TCCR1A |= (1 << WGM11);

	/* Clock cu prescalar 1024 */
	TCCR1B |= (1 << CS10);
	TCCR1B |= (1 << CS12);
	
	// NOTE : 15625 = 1 secunda
	TCNT1 = 0;
}

void blink_led_test(float dist)
{
	while(dist > 0) {
		PORTD |= (1 << PD7);
		_delay_ms(1000);
		PORTD &= ~(1 << PD7);
		_delay_ms(1000);
		dist -= 1;		
	}
}

void error_blink()
{
	while(1) {
		PORTD ^= (1 << PD7);
		_delay_ms(60);	
	}
}

void servo_test()
{
	uint8_t i;
	while(1) {
		// Misc servo pana la -90
		for(i=0 ; i< 15;i++){
		    PORTC |= (1 << PC0);
		    _delay_ms(0.6);
		    PORTC &= ~(1 << PC0);
		    _delay_ms(30);
		}
		_delay_ms(1000);
		// Misc servo inapoi la 90 
		for(i=0 ; i < 15 ; i++){
		    PORTC |= (1 << PC0);
		    _delay_ms(2.5);
		    PORTC &= ~(1 << PC0);
		    _delay_ms(18);
		}
		_delay_ms(2000);
	}
}

void sensor_test()
{
	bool stop = false;
	unsigned int time = 0;	
	float micro_sec = 0;
	float dist = 0;
	
	// Setez TRIG pe 0 just to be sure
	PORTA &= ~(1 << PA1);
	_delay_ms(2);
	// Setez TRIG pe 1
	PORTA |= (1 << PA1);
	_delay_ms(10);
	PORTA &= ~(1 << PA1);
	
	while (1) {
		// Daca primesc input pe ECHO
		if ((PINA & (1 << PA0)) && (stop == false)) {
			// Masor timpul pana ECHO trece pe low
			stop = true;
			TCNT1 = 0;
		// ECHO a trecut pe low, retin timpul
		} else if (!(PINA & (1 << PA0)) && (stop == true)) {
			time = TCNT1;
			break;	
		}
	}
	// Check
	if (time == 0)
		error_blink();

	micro_sec = time * 1000000 / 15625;
	// Check
	if (micro_sec == 0)
		error_blink();
	// In centimeters
	dist = micro_sec * 0.034 / 2;
	// Check
	if (dist == 0)
		error_blink();

	// Everything is ok. Blink led to notify
	PORTD |= (1 << PD7);
	_delay_ms(2000);
	PORTD &= ~(1 << PD7);
	_delay_ms(50);
	
	// Blink led for every centimeter in dist
	blink_led_test(dist);

}

void start()
{
	uint8_t i;
	bool stop = false;
	unsigned int time = 0;	
	float micro_sec = 0;
	float dist = 0;

	while(1) {
		// Misc servo pana la -90
		for(i=0 ; i< 10;i++) {
			PORTC |= (1 << PC0);
			_delay_ms(0.6);
			PORTC &= ~(1 << PC0);

			/* Masor cu senzorul */
			// Setez TRIG pe 0 just to be sure
			PORTA &= ~(1 << PA1);
			_delay_ms(2);
			// Setez TRIG pe 1
			PORTA |= (1 << PA1);
			_delay_ms(10);
			PORTA &= ~(1 << PA1);
	
			while (1) {
				// Daca primesc input pe ECHO
				if ((PINA & (1 << PA0)) && (stop == false)) {
					// Masor timpul pana ECHO trece pe low
					stop = true;
					TCNT1 = 0;
				// ECHO a trecut pe low, retin timpul
				} else if (!(PINA & (1 << PA0)) && (stop == true)) {
					time = TCNT1;
					break;	
				}
			}
			
			// Check
			if (time == 0)
				error_blink();

			micro_sec = time * 1000000 / 15625;
			// Check
			if (micro_sec == 0)
				error_blink();
			// In centimeters
			dist = micro_sec * 0.034 / 2;
			// Check
			if (dist == 0)
				error_blink();

			// Everything is ok. Blink led to notify
			PORTD |= (1 << PD7);
			_delay_ms(10);
			PORTD &= ~(1 << PD7);
			_delay_ms(10);

			// TODO : Send info via USART

			// Random delay for servo
			//_delay_ms(60);

		}
		_delay_ms(1000);
		// Misc servo inapoi la 90 
		for(i=0 ; i < 15 ; i++){
		    PORTC |= (1 << PC0);
		    _delay_ms(2.5);
		    PORTC &= ~(1 << PC0);
		    _delay_ms(18);
		}
		_delay_ms(2000);
	}

}

int main(void) {
	IO_init();	
	timer1_init();

	#if defined(SERVO_TEST)
	servo_test();
	#elif defined(SENSOR_TEST)
	sensor_test();
	#else
	start();
	#endif

	return 0;
}
